/****************************************************************************
 *
 * Copyright 2019 Samsung Electronics All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific
 * language governing permissions and limitations under the License.
 *
 ****************************************************************************/
/****************************************************************************
 * apps/examples/heavy_signal_message_test/heavy_signal_message_test_stop.c
 *
 *   Copyright (C) 2008, 2011-2012 Gregory Nutt. All rights reserved.
 *   Author: Gregory Nutt <gnutt@nuttx.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name NuttX nor the names of its contributors may be
 *    used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

/****************************************************************************
 * Included Files
 ****************************************************************************/

#include <tinyara/config.h>
#include <stdlib.h>
#include <fcntl.h>
#include <mqueue.h>
#include <string.h>

#include "heavy_signal_message_test.h"

void heavy_signal_message_test_stop(void)
{
	mqd_t send_fd;
	mqd_t recv_fd;
	int command;
	struct mq_attr attr;
	char send_q_name[HEAVY_SIGNAL_MESSAGE_TEST_MQ_NAME_MAX];
	char recv_q_name[HEAVY_SIGNAL_MESSAGE_TEST_MQ_NAME_MAX];

	attr.mq_maxmsg = HEAVY_SIGNAL_MESSAGE_TEST_MAX_MSG;
	attr.mq_msgsize = sizeof(int);
	attr.mq_flags = 0;

	strncpy(send_q_name, HEAVY_SIGNAL_MESSAGE_TEST_SEND_STOP_MQ, HEAVY_SIGNAL_MESSAGE_TEST_MQ_NAME_MAX);
	strncpy(recv_q_name, HEAVY_SIGNAL_MESSAGE_TEST_RECEIVE_STOP_MQ, HEAVY_SIGNAL_MESSAGE_TEST_MQ_NAME_MAX);

	recv_fd = mq_open(recv_q_name, O_WRONLY | O_CREAT, 0666, &attr);
	if (recv_fd < 0) {
		printf("Failed to open message queue\n");
		return;
	}

	send_fd = mq_open(send_q_name, O_WRONLY | O_CREAT, 0666, &attr);
	if (send_fd < 0) {
		printf("Failed to open message queue\n");
		mq_close(recv_fd);
		mq_unlink(recv_q_name);
		return;
	}

	command = HEAVY_SIGNAL_MESSAGE_TEST_STOP;
	mq_send(recv_fd, (char *)&command, sizeof(int), 100);
	mq_send(send_fd, (char *)&command, sizeof(int), 100);

	mq_close(send_fd);
	mq_close(recv_fd);

	return;
}
