/****************************************************************************
 *
 * Copyright 2018 Samsung Electronics All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific
 * language governing permissions and limitations under the License.
 *
 ****************************************************************************/
//===----------------------------------------------------------------------===//
//
//                     The LLVM Compiler Infrastructure
//
// This file is dual licensed under the MIT and the University of Illinois Open
// Source Licenses. See LICENSE.TXT for details.
//
//===----------------------------------------------------------------------===//

// <string>

// int compare(size_type pos1, size_type n1, const basic_string& str,
//             size_type pos2, size_type n2=npos) const;
//  the "=npos" was added in C++14

#include <string>
#include <stdexcept>
#include <cassert>
#include "libcxx_tc_common.h"


#include "test_macros.h"

static int sign(int x)
{
    if (x == 0)
        return 0;
    if (x < 0)
        return -1;
    return 1;
}

template <class S>
static int
test(const S& s,   typename S::size_type pos1, typename S::size_type n1,
     const S& str, typename S::size_type pos2, typename S::size_type n2, int x)
{
    if (pos1 <= s.size() && pos2 <= str.size())
    {
        TC_ASSERT_EXPR(sign(s.compare(pos1, n1, str, pos2, n2)) == sign(x));
    }
#ifndef TEST_HAS_NO_EXCEPTIONS
    else
    {
        try
        {
            TEST_IGNORE_NODISCARD s.compare(pos1, n1, str, pos2, n2);
            TC_ASSERT_EXPR(false);
        }
        catch (const std::out_of_range&)
        {
            TC_ASSERT_EXPR(pos1 > s.size() || pos2 > str.size());
        }
    }
#endif
    return 0;
}

template <class S>
static int
test_npos(const S& s,   typename S::size_type pos1, typename S::size_type n1,
          const S& str, typename S::size_type pos2, int x)
{
    if (pos1 <= s.size() && pos2 <= str.size())
    {
        TC_ASSERT_EXPR(sign(s.compare(pos1, n1, str, pos2)) == sign(x));
    }
#ifndef TEST_HAS_NO_EXCEPTIONS
    else
    {
        try
        {
            TEST_IGNORE_NODISCARD s.compare(pos1, n1, str, pos2);
            TC_ASSERT_EXPR(false);
        }
        catch (const std::out_of_range&)
        {
            TC_ASSERT_EXPR(pos1 > s.size() || pos2 > str.size());
        }
    }
#endif
    return 0;
}

template <class S>
static int test0()
{
    TC_ASSERT_FUNC((test(S(""), 0, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcde"), 6, 0, 0)));
    return 0;
}

template <class S>
static int test1()
{
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 0, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 0, 2, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 0, 4, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 0, 5, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 0, 6, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 1, 2, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 1, 3, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 1, 4, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 1, 5, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 2, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 2, 2, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 2, 3, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 2, 4, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 4, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 4, 2, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 0, 5, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 0, 9, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 0, 10, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 0, 11, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 1, 4, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 1, 8, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 1, 9, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 1, 10, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 5, 2, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 5, 4, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 5, 5, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 5, 6, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 9, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 9, 2, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 0, 1, 0)));
    return 0;
}

template <class S>
static int test2()
{
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 0, 10, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 0, 19, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 0, 20, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 0, 21, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 1, 9, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 1, 18, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 1, 19, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 1, 20, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 10, 5, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 10, 9, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 10, 10, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 10, 11, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 19, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 19, 2, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S(""), 1, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S(""), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 0, 0, 1)));
    return 0;
}

template <class S>
static int test3()
{
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 0, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 0, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 0, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 0, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 2, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 0, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 0, 9, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 0, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 0, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 0, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 0, 19, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 0, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 0, 21, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S(""), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S(""), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 0, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 0, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 0, 5, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 0, 6, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 1, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 2, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 4, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 5, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 5, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 0, 5, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 0, 9, -7)));
    return 0;
}

template <class S>
static int test4()
{
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 0, 10, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 0, 11, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 1, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 5, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 9, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 10, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 10, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 0, 10, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 0, 19, -17)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 0, 20, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 0, 21, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 1, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 10, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 19, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 20, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 20, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 2, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S(""), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S(""), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 0, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 0, 2, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 0, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 0, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 0, 6, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 2, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 4, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 5, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 5, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 0, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 0, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 0, 9, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 0, 10, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 0, 11, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 5, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 9, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 10, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 10, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 0, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 0, 10, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 0, 19, -15)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 0, 20, -16)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 0, 21, -16)));
    return 0;
}

template <class S>
static int test5()
{
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 10, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 19, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 20, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 20, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 4, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S(""), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S(""), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 0, 2, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 0, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 0, 6, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 2, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 4, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 5, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 5, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 0, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 0, 9, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 0, 10, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 0, 11, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 5, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 9, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 10, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 10, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 0, 10, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 0, 19, -14)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 0, 20, -15)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 0, 21, -15)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 10, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 19, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 20, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 20, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 5, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S(""), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S(""), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 0, 2, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 0, 5, 0)));
    return 0;
}

template <class S>
static int test6()
{
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 0, 6, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 2, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 4, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 5, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 5, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 0, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 0, 9, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 0, 10, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 0, 11, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 5, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 9, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 10, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 10, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 0, 10, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 0, 19, -14)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 0, 20, -15)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 0, 21, -15)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 10, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 19, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 20, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 20, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcde"), 0, 6, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 1, 1, -1)));
    return 0;
}

template <class S>
static int test7()
{
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S(""), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 1, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 1, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 1, 5, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 2, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 1, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 1, 8, -7)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 1, 9, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 1, 10, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 1, 9, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 1, 18, -17)));
    return 0;
}

template <class S>
static int test8()
{
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 1, 19, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 1, 20, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S(""), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S(""), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 1, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 1, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 1, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 1, 5, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 2, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 4, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 5, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 5, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 1, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 1, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 1, 8, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 1, 9, -7)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 1, 10, -7)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 5, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 9, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 10, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 10, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 1, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 1, 9, -7)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 1, 18, -16)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 1, 19, -17)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 1, 20, -17)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 10, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 19, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 20, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 20, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 2, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S(""), 0, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S(""), 0, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 0, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 1, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 1, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 1, 2, 1)));
    return 0;
}

template <class S>
static int test9()
{
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 1, 3, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 2, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 4, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 5, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 5, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 0, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 1, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 1, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 1, 8, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 1, 9, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 1, 10, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 5, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 9, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 10, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 10, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 0, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 1, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 1, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 1, 9, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 1, 18, -15)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 1, 19, -16)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 1, 20, -16)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 10, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 19, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 20, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 20, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 3, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S(""), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S(""), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 1, 2, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 1, 3, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 1, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 1, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 2, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 4, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 5, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 5, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 1, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 1, 8, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 1, 9, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 1, 10, -5)));
    return 0;
}

template <class S>
static int test10()
{
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 5, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 9, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 10, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 10, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 1, 9, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 1, 18, -14)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 1, 19, -15)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 1, 20, -15)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 10, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 19, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 20, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 20, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 4, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S(""), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S(""), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 1, 2, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 1, 3, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 1, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 1, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 2, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 4, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 5, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 5, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 1, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 1, 8, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 1, 9, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 1, 10, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 5, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 9, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 10, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 10, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 1, 9, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 1, 18, -14)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 1, 19, -15)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 1, 20, -15)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 10, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    return 0;
}

template <class S>
static int test11()
{
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 19, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 20, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 20, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 1, 5, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S(""), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 0, 2, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 0, 4, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 0, 5, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 0, 6, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 1, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 1, 3, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 1, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 1, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 2, 0, 1)));
    return 0;
}

template <class S>
static int test12()
{
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 2, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 4, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 4, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 0, 5, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 0, 9, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 0, 10, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 0, 11, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 1, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 1, 8, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 1, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 1, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 5, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 5, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 5, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 5, 5, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 5, 6, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 9, 1, -7)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 9, 2, -7)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 0, 10, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 0, 19, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 0, 20, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 0, 21, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 1, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 1, 18, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 1, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 1, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 10, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 10, 5, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 10, 9, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 10, 10, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 10, 11, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 19, 1, -17)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 19, 2, -17)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S(""), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S(""), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 0, 2, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 0, 4, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 0, 5, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 0, 6, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 1, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 1, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 1, 3, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 1, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 1, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 2, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 2, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 2, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 4, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 4, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 4, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 5, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 5, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 0, 5, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 0, 9, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 0, 10, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 0, 11, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 1, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 1, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 1, 8, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 1, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 1, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 5, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 5, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 5, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 5, 4, -3)));
    return 0;
}

template <class S>
static int test13()
{
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 5, 5, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 5, 6, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 9, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 9, 1, -7)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 9, 2, -7)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 10, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 10, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 0, 10, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 0, 19, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 0, 20, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 0, 21, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 1, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 1, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 1, 18, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 1, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 1, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 10, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 10, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 10, 5, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 10, 9, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 10, 10, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 10, 11, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 19, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 19, 1, -17)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 19, 2, -17)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 20, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 20, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 2, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S(""), 0, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S(""), 0, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 0, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 0, 2, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 0, 4, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 0, 5, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 0, 6, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 1, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 1, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 1, 3, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 1, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 1, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 2, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 2, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 2, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 2, 3, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 2, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 4, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 4, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 4, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 5, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 5, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 0, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 0, 5, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 0, 9, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 0, 10, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 0, 11, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 1, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 1, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 1, 8, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 1, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 1, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 5, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 5, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 5, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 5, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 5, 5, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 5, 6, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 9, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 9, 1, -7)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 9, 2, -7)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 10, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 10, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 0, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 0, 10, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 0, 19, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 0, 20, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 0, 21, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 1, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 1, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 1, 18, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 1, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 1, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 10, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 10, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 10, 5, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 10, 9, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 10, 10, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 10, 11, -8)));
    return 0;
}

template <class S>
static int test14()
{
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 19, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 19, 1, -17)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 19, 2, -17)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 20, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 20, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 3, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S(""), 0, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S(""), 0, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 0, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 0, 2, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 0, 4, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 0, 5, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 0, 6, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 1, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 1, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 1, 3, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 1, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 1, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 2, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 2, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 2, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 2, 3, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 2, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 4, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 4, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 4, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 5, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 5, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 0, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 0, 5, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 0, 9, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 0, 10, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 0, 11, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 1, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 1, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 1, 8, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 1, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 1, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 5, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 5, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 5, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 5, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 5, 5, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 5, 6, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 9, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 9, 1, -7)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 9, 2, -7)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 10, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 10, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 0, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 0, 10, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 0, 19, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 0, 20, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 0, 21, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 1, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 1, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 1, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 1, 18, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 1, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 1, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 10, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 10, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 10, 5, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 10, 9, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 10, 10, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 10, 11, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 19, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 19, 1, -17)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 19, 2, -17)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 20, 0, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 20, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 2, 4, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 2, 4, -3)));
    return 0;
}

template <class S>
static int test15()
{
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S(""), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 0, 2, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 0, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 0, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 0, 6, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 1, 2, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 1, 3, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 1, 4, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 1, 5, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 2, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 2, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 2, 2, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 2, 3, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 2, 4, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 4, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 4, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 0, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 0, 9, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 0, 10, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 0, 11, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 1, 4, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 1, 8, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 1, 9, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 1, 10, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 5, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 5, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 5, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 5, 6, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 9, 1, -5)));
    return 0;
}

template <class S>
static int test16()
{
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 9, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 0, 10, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 0, 19, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 0, 20, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 0, 21, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 1, 9, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 1, 18, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 1, 19, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 1, 20, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 10, 1, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 10, 5, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 10, 9, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 10, 10, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 10, 11, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 19, 1, -15)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 19, 2, -15)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S(""), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 0, 2, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 0, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 0, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 0, 6, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 1, 2, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 1, 3, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 1, 4, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 1, 5, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 2, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 2, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 2, 2, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 2, 3, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 2, 4, 2)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 4, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 4, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 0, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 0, 9, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 0, 10, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 0, 11, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 1, 4, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 1, 8, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 1, 9, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 1, 10, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 5, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 5, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 5, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 5, 6, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 9, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 9, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 0, 10, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 0, 19, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 0, 20, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 0, 21, 4)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 1, 9, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 1, 18, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 1, 19, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 1, 20, 3)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 10, 1, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 10, 5, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 10, 9, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 10, 10, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 10, 11, -6)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 19, 1, -15)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 19, 2, -15)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    return 0;
}

template <class S>
static int test17()
{
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcde"), 4, 2, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 5, 0, 0)));
    return 0;
}

template <class S>
static int test18()
{
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 5, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 0, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 0, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 0, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 0, 6, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 1, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 1, 3, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 1, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 1, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 2, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 2, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 2, 3, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 2, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 4, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 4, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 0, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 0, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 0, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 0, 11, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 1, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 1, 8, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 1, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 1, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 5, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 5, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 5, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 5, 6, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 9, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 9, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghij"), 11, 0, 0)));
    return 0;
}

template <class S>
static int test19()
{
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 0, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 0, 19, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 0, 20, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 0, 21, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 1, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 1, 18, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 1, 19, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 1, 20, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 10, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 10, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 10, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 10, 11, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 19, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 19, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcde"), 6, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S(""), 0, 1, 1)));
    return 0;
}

template <class S>
static int test20()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 0, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 0, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 0, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 0, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 2, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 0, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 0, 9, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 0, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 0, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 0, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 0, 19, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 0, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 0, 21, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S(""), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S(""), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 0, 2, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 0, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 0, 6, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 2, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 4, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 5, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 5, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 0, 1, 4)));
    return 0;
}

template <class S>
static int test21()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 0, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 0, 9, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 0, 10, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 0, 11, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 5, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 9, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 10, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 10, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 0, 10, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 0, 19, -14)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 0, 20, -15)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 0, 21, -15)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 10, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 19, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 20, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 20, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 5, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S(""), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S(""), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 0, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 0, 2, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 0, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 0, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 0, 6, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 2, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 4, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 5, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 5, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 0, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 0, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 0, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 0, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 0, 11, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 5, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 9, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 10, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 10, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 0, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 0, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 0, 19, -10)));
    return 0;
}

template <class S>
static int test22()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 0, 20, -11)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 0, 21, -11)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 10, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 19, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 20, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 20, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 9, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S(""), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S(""), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 0, 2, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 0, 4, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 0, 6, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 2, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 4, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 5, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 5, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 0, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 0, 11, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 5, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 9, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 10, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 10, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 0, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 0, 19, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 0, 20, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 0, 21, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 10, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 19, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 20, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 20, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 10, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S(""), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S(""), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 0, 2, 8)));
    return 0;
}

template <class S>
static int test23()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 0, 4, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 0, 6, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 2, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 4, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 5, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 5, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 0, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 0, 11, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 5, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 9, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 10, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 10, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 0, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 0, 19, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 0, 20, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 0, 21, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 10, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 19, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 20, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 20, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 0, 11, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 0, 11, -10)));
    return 0;
}

template <class S>
static int test24()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S(""), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 1, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 1, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 1, 5, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 2, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 1, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 1, 8, -7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 1, 9, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 1, 10, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 1, 1, 0)));
    return 0;
}

template <class S>
static int test25()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 1, 9, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 1, 18, -17)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 1, 19, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 1, 20, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S(""), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S(""), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 1, 2, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 1, 3, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 1, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 1, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 2, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 4, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 5, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 5, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 1, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 1, 8, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 1, 9, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 1, 10, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 5, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 9, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 10, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 10, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 1, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 1, 9, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 1, 18, -14)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 1, 19, -15)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 1, 20, -15)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 10, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 19, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 20, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 20, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 4, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S(""), 0, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S(""), 0, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 0, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 1, 0, 8)));
    return 0;
}

template <class S>
static int test26()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 1, 1, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 1, 2, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 1, 3, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 1, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 1, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 2, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 4, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 5, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 5, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 0, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 1, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 1, 1, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 1, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 1, 8, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 5, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 9, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 10, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 10, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 0, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 1, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 1, 1, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 1, 18, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 1, 19, -11)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 1, 20, -11)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 10, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 19, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 20, 0, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 20, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 8, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S(""), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S(""), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 1, 2, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 1, 3, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 1, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 1, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 2, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 4, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 5, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 5, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 1, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 1, 8, 1)));
    return 0;
}

template <class S>
static int test27()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 1, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 1, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 5, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 9, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 10, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 10, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 1, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 1, 18, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 1, 19, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 1, 20, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 10, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 19, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 20, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 20, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 9, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S(""), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S(""), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 1, 2, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 1, 3, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 1, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 1, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 2, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 4, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 5, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 5, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 1, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 1, 8, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 1, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 1, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 5, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 9, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 10, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 10, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 1, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 1, 18, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 1, 19, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 1, 20, -10)));
    return 0;
}

template <class S>
static int test28()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 10, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 19, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 20, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 20, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 1, 10, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S(""), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 0, 2, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 0, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 0, 6, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 1, 2, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 1, 3, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 1, 4, 4)));
    return 0;
}

template <class S>
static int test29()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 1, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 2, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 2, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 2, 2, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 2, 3, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 2, 4, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 4, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 4, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 0, 9, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 0, 10, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 0, 11, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 1, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 1, 8, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 1, 9, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 1, 10, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 5, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 5, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 9, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 9, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 0, 10, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 0, 19, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 0, 20, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 0, 21, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 1, 9, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 1, 18, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 1, 19, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 1, 20, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 10, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 10, 9, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 10, 10, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 10, 11, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 19, 1, -14)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 19, 2, -14)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S(""), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S(""), 0, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 0, 2, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 0, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 0, 6, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 1, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 1, 2, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 1, 3, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 1, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 1, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 2, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 2, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 2, 2, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 2, 3, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 2, 4, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 4, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 4, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 4, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 5, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 5, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 0, 9, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 0, 10, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 0, 11, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 1, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 1, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 1, 8, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 1, 9, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 1, 10, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 5, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 5, 1, 1)));
    return 0;
}

template <class S>
static int test30()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 5, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 5, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 5, 5, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 5, 6, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 9, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 9, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 9, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 10, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 10, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 0, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 0, 10, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 0, 19, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 0, 20, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 0, 21, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 1, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 1, 9, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 1, 18, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 1, 19, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 1, 20, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 10, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 10, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 10, 9, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 10, 10, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 10, 11, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 19, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 19, 1, -14)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 19, 2, -14)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 20, 0, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 20, 1, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 2, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S(""), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S(""), 0, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 0, 2, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 0, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 0, 6, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 1, 2, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 1, 3, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 1, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 1, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 2, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 2, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 2, 2, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 2, 3, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 2, 4, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 4, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 4, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 4, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 5, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 5, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 0, 9, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 0, 10, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 0, 11, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 1, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 1, 8, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 1, 9, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 1, 10, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 5, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 5, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 5, 2, 2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 5, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 5, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 5, 6, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 9, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 9, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 9, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 10, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 10, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 0, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 0, 10, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 0, 19, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 0, 20, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 0, 21, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 1, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 1, 9, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 1, 18, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 1, 19, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 1, 20, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 10, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 10, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 10, 9, -5)));
    return 0;
}

template <class S>
static int test31()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 10, 10, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 10, 11, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 19, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 19, 1, -14)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 19, 2, -14)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 20, 0, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 20, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 4, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S(""), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S(""), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 0, 2, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 0, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 0, 6, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 1, 2, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 1, 3, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 1, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 1, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 2, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 2, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 2, 2, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 2, 3, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 2, 4, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 4, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 4, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 4, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 5, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 5, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 0, 9, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 0, 10, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 0, 11, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 1, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 1, 8, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 1, 9, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 1, 10, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 5, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 5, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 5, 2, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 5, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 5, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 5, 6, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 9, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 9, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 9, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 10, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 10, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 0, 10, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 0, 19, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 0, 20, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 0, 21, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 1, 9, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 1, 18, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 1, 19, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 1, 20, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 10, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 10, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 10, 9, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 10, 10, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 10, 11, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 19, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 19, 1, -14)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 19, 2, -14)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 20, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 20, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 5, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S(""), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S(""), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 0, 2, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 0, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 0, 6, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 1, 2, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 1, 3, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 1, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 1, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 2, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 2, 1, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 2, 2, 3)));
    return 0;
}

template <class S>
static int test32()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 2, 3, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 2, 4, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 4, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 4, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 4, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 5, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 5, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 0, 9, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 0, 10, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 0, 11, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 1, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 1, 8, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 1, 9, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 1, 10, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 5, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 5, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 5, 2, 3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 5, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 5, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 5, 6, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 9, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 9, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 9, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 10, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 10, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 0, 10, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 0, 19, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 0, 20, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 0, 21, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 1, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 1, 9, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 1, 18, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 1, 19, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 1, 20, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 10, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 10, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 10, 9, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 10, 10, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 10, 11, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 19, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 19, 1, -14)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 19, 2, -14)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 20, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 20, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 5, 6, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 5, 6, -5)));
    return 0;
}

template <class S>
static int test33()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S(""), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 0, 2, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 0, 4, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 0, 5, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 0, 6, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 1, 2, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 1, 3, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 1, 4, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 1, 5, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 2, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 2, 1, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 2, 2, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 2, 3, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 2, 4, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 4, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 4, 2, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 0, 5, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 0, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 0, 10, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 0, 11, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 1, 4, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 1, 8, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 1, 9, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 1, 10, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 5, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 5, 2, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 5, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 5, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 5, 6, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 9, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 9, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 0, 10, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 0, 19, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 0, 20, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 0, 21, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 1, 9, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 1, 18, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 1, 19, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 1, 20, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 10, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 10, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 10, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 10, 11, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 19, 1, -10)));
    return 0;
}

template <class S>
static int test34()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 19, 2, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S(""), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 0, 2, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 0, 4, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 0, 5, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 0, 6, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 1, 2, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 1, 3, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 1, 4, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 1, 5, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 2, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 2, 1, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 2, 2, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 2, 3, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 2, 4, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 4, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 4, 2, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 0, 5, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 0, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 0, 10, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 0, 11, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 1, 4, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 1, 8, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 1, 9, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 1, 10, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 5, 1, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 5, 2, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 5, 4, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 5, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 5, 6, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 9, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 9, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 0, 10, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 0, 19, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 0, 20, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 0, 21, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 1, 9, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 1, 18, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 1, 19, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 1, 20, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 10, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 10, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 10, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 10, 11, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 19, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 19, 2, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 9, 2, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 4, 1, -1)));
    return 0;
}

template <class S>
static int test35()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 10, 0, 0)));
    return 0;
}

template <class S>
static int test36()
{
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 10, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 0, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 0, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 0, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 0, 6, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 1, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 1, 3, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 1, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 1, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 2, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 2, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 2, 3, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 2, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 4, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 4, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 0, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 0, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 0, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 0, 11, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 1, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 1, 8, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 1, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 1, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 5, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 5, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 5, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 5, 6, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 9, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 9, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 0, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 0, 19, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 0, 20, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 0, 21, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 1, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 1, 18, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 1, 19, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 1, 20, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 10, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 10, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 10, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 10, 11, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 19, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 19, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghij"), 11, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    return 0;
}

template <class S>
static int test37()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S(""), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 0, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 0, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 0, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 0, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 2, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcde"), 6, 0, 0)));
    return 0;
}

template <class S>
static int test38()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 0, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 0, 9, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 0, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 0, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 0, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 0, 19, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 0, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 0, 21, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S(""), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S(""), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 0, 2, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 0, 4, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 0, 6, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 2, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 4, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 5, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 5, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 0, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 0, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 0, 11, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 5, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 9, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 10, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 10, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 0, 1, 9)));
    return 0;
}

template <class S>
static int test39()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 0, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 0, 19, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 0, 20, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 0, 21, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 10, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 19, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 20, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 20, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 10, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S(""), 0, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S(""), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 0, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 0, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 0, 2, 17)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 0, 4, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 0, 5, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 0, 6, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 1, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 2, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 4, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 5, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 5, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 0, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 0, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 0, 5, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 0, 9, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 0, 10, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 0, 11, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 1, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 5, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 9, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 10, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 10, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 0, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 0, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 0, 10, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 0, 19, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 0, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 0, 21, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 1, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 10, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 19, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 20, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 20, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 19, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S(""), 0, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S(""), 0, 1, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 0, 0, 20)));
    return 0;
}

template <class S>
static int test40()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 0, 2, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 0, 4, 16)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 0, 5, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 0, 6, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 1, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 2, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 4, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 5, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 5, 1, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 0, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 0, 5, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 0, 9, 11)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 0, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 0, 11, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 1, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 5, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 9, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 10, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 10, 1, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 0, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 0, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 0, 20, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 0, 21, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 1, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 10, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 19, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 20, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 20, 1, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 20, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S(""), 0, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S(""), 0, 1, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 0, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 0, 2, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 0, 4, 16)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 0, 5, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 0, 6, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 1, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 1, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 1, 5, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 2, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 2, 1, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 2, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 2, 4, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 4, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 4, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 4, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 5, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 5, 1, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 0, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 0, 5, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 0, 9, 11)));
    return 0;
}

template <class S>
static int test41()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 0, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 0, 11, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 1, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 1, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 1, 8, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 1, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 5, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 5, 1, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 5, 2, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 5, 4, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 9, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 9, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 9, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 10, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 10, 1, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 0, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 0, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 0, 20, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 0, 21, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 1, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 1, 9, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 1, 18, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 10, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 10, 1, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 10, 5, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 10, 9, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 19, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 19, 1, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 19, 2, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 20, 0, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 20, 1, 20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 0, 21, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    return 0;
}

template <class S>
static int test42()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S(""), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 1, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 1, 3, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 1, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 1, 5, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 2, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 1, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 1, 8, -7)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 1, 9, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 1, 10, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 1, 9, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 1, 18, -17)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 1, 19, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 1, 20, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S(""), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S(""), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 0, 5, 1)));
    return 0;
}

template <class S>
static int test43()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 1, 2, 7)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 1, 3, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 1, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 1, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 2, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 4, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 5, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 5, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 1, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 1, 8, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 1, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 1, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 5, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 9, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 10, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 10, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 1, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 1, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 1, 18, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 1, 19, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 1, 20, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 10, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 19, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 20, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 20, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 9, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S(""), 0, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S(""), 0, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 0, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 1, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 1, 1, 17)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 1, 2, 16)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 1, 3, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 1, 4, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 1, 5, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 2, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 4, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 5, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 5, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 0, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 1, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 1, 1, 17)));
    return 0;
}

template <class S>
static int test44()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 1, 4, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 1, 8, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 1, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 1, 10, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 5, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 9, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 10, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 10, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 0, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 1, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 1, 1, 17)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 1, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 1, 18, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 1, 19, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 1, 20, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 10, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 19, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 20, 0, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 20, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 18, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S(""), 0, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S(""), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 0, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 1, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 1, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 1, 2, 17)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 1, 3, 16)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 1, 4, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 1, 5, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 2, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 4, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 5, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 5, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 0, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 1, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 1, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 1, 4, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 1, 8, 11)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 1, 9, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 1, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 5, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 9, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 10, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 10, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 0, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 1, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 1, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 1, 9, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 1, 18, 1)));
    return 0;
}

template <class S>
static int test45()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 1, 19, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 1, 20, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 10, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 19, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 20, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 20, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 19, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S(""), 0, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S(""), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 0, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 0, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 0, 4, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 0, 6, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 1, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 1, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 1, 2, 17)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 1, 3, 16)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 1, 4, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 1, 5, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 2, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 2, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 2, 3, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 2, 4, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 4, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 4, 1, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 4, 2, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 5, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 5, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 0, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 0, 5, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 0, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 0, 11, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 1, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 1, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 1, 4, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 1, 8, 11)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 1, 9, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 1, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 5, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 5, 1, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 5, 2, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 5, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 5, 6, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 9, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 9, 1, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 9, 2, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 10, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 10, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 0, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 0, 10, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 0, 19, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 0, 20, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 0, 21, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 1, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 1, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 1, 9, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 1, 18, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 1, 19, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 1, 20, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 10, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 10, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 10, 5, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 19, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 19, 1, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 19, 2, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 20, 0, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 20, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 1, 20, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 1, 2, -2)));
    return 0;
}

template <class S>
static int test46()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S(""), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 0, 2, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 0, 4, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 0, 5, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 0, 6, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 1, 2, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 1, 3, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 1, 4, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 1, 5, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 2, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 2, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 2, 2, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 2, 3, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 2, 4, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 4, 1, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 4, 2, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 0, 5, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 0, 9, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 0, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 0, 11, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 1, 4, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 1, 8, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 1, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 1, 10, 9)));
    return 0;
}

template <class S>
static int test47()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 5, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 5, 2, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 5, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 5, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 5, 6, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 9, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 9, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 0, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 0, 19, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 0, 20, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 0, 21, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 1, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 1, 18, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 1, 19, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 1, 20, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 10, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 10, 9, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 10, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 10, 11, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 19, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 19, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S(""), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S(""), 0, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 0, 2, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 0, 4, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 0, 5, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 0, 6, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 1, 2, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 1, 3, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 1, 4, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 1, 5, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 2, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 2, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 2, 2, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 2, 3, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 2, 4, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 4, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 4, 1, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 4, 2, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 5, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 5, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 0, 5, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 0, 9, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 0, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 0, 11, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 1, 4, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 1, 8, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 1, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 1, 10, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 5, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 5, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 5, 2, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 5, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 5, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 5, 6, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 9, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 9, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 9, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 10, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 10, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 0, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 0, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 0, 19, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 0, 20, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 0, 21, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 1, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 1, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 1, 18, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 1, 19, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 1, 20, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 10, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 10, 1, 4)));
    return 0;
}

template <class S>
static int test48()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 10, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 10, 9, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 10, 10, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 10, 11, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 19, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 19, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 19, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 20, 0, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 20, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 5, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S(""), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S(""), 0, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 0, 2, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 0, 4, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 0, 5, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 0, 6, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 1, 2, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 1, 3, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 1, 4, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 1, 5, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 2, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 2, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 2, 2, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 2, 3, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 2, 4, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 4, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 4, 1, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 4, 2, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 5, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 5, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 0, 5, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 0, 9, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 0, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 0, 11, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 1, 4, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 1, 8, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 1, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 1, 10, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 5, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 5, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 5, 2, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 5, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 5, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 5, 6, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 9, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 9, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 9, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 10, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 10, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 0, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 0, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 0, 19, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 0, 20, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 0, 21, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 1, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 1, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 1, 18, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 1, 19, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 1, 20, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 10, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 10, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 10, 5, 4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 10, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 10, 10, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 10, 11, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 19, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 19, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 19, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 20, 0, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 20, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 9, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S(""), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S(""), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 0, 2, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 0, 4, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 0, 5, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 0, 6, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 1, 2, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 1, 3, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 1, 4, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 1, 5, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 2, 0, 10)));
    return 0;
}

template <class S>
static int test49()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 2, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 2, 2, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 2, 3, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 2, 4, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 4, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 4, 1, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 4, 2, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 5, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 5, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 0, 5, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 0, 9, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 0, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 0, 11, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 1, 4, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 1, 8, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 1, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 1, 10, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 5, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 5, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 5, 2, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 5, 4, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 5, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 5, 6, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 9, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 9, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 9, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 10, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 10, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 0, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 0, 19, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 0, 20, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 0, 21, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 1, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 1, 18, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 1, 19, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 1, 20, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 10, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 10, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 10, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 10, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 10, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 10, 11, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 19, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 19, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 19, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 20, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 20, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 10, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S(""), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S(""), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 0, 2, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 0, 4, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 0, 5, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 0, 6, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 1, 2, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 1, 3, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 1, 4, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 1, 5, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 2, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 2, 1, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 2, 2, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 2, 3, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 2, 4, 8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 4, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 4, 1, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 4, 2, 6)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 5, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 5, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 0, 5, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 0, 9, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 0, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 0, 11, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 1, 4, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 1, 8, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 1, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 1, 10, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 5, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 5, 1, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 5, 2, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 5, 4, 5)));
    return 0;
}

template <class S>
static int test50()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 5, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 5, 6, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 9, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 9, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 9, 2, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 10, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 10, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 0, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 0, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 0, 10, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 0, 19, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 0, 20, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 0, 21, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 1, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 1, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 1, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 1, 18, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 1, 19, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 1, 20, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 10, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 10, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 10, 5, 5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 10, 9, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 10, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 10, 11, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 19, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 19, 1, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 19, 2, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 20, 0, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 20, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 10, 11, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    return 0;
}

template <class S>
static int test51()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S(""), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 0, 2, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 0, 4, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 0, 5, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 0, 6, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 1, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 1, 2, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 1, 3, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 1, 4, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 1, 5, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 2, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 2, 1, 17)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 2, 2, 17)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 2, 3, 17)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 2, 4, 17)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 4, 1, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 4, 2, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 0, 5, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 0, 9, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 0, 10, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 0, 11, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 1, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 1, 4, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 1, 8, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 1, 9, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 1, 10, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 5, 1, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 5, 2, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 5, 4, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 5, 5, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 5, 6, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 9, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 9, 2, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 0, 10, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 0, 19, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 0, 20, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 0, 21, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 1, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 1, 9, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 1, 18, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 1, 19, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 1, 20, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 10, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 10, 5, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 10, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 10, 10, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 10, 11, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 19, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 19, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S(""), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S(""), 0, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 0, 2, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 0, 4, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 0, 5, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 0, 6, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 1, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 1, 2, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 1, 3, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 1, 4, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 1, 5, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 2, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 2, 1, 17)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 2, 2, 17)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 2, 3, 17)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 2, 4, 17)));
    return 0;
}

template <class S>
static int test52()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 4, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 4, 1, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 4, 2, 15)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 5, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 0, 5, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 0, 9, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 0, 10, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 0, 11, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 1, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 1, 4, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 1, 8, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 1, 9, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 1, 10, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 5, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 5, 1, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 5, 2, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 5, 4, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 5, 5, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 5, 6, 14)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 9, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 9, 1, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 9, 2, 10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 10, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 0, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 0, 1, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 0, 10, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 0, 19, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 0, 20, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 0, 21, 19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 1, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 1, 1, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 1, 9, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 1, 18, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 1, 19, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 1, 20, 18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 10, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 10, 1, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 10, 5, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 10, 9, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 10, 10, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 10, 11, 9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 19, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 19, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 19, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 20, 0, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 20, 1, 1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 19, 2, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 9, 1, -1)));
    return 0;
}

template <class S>
static int test53()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 0, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 0, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 0, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 1, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 1, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 1, 5, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 2, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 2, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 2, 3, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 2, 4, -3)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 4, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 4, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 0, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 0, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 0, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 1, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 1, 8, -8)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 1, 10, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 5, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 5, 2, -2)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 5, 4, -4)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 5, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 5, 6, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 9, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 9, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 0, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 0, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 0, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 0, 20, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 0, 21, -20)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 1, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 1, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 1, 18, -18)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 1, 19, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 1, 20, -19)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 10, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 10, 5, -5)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 10, 9, -9)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 10, 10, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 10, 11, -10)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 19, 1, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 19, 2, -1)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    return 0;
}

template <class S>
static int test54()
{
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 20, 1, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S(""), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S(""), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S(""), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 0, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 0, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 0, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 0, 6, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 1, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 1, 3, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 1, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 1, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 2, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 2, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 2, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 2, 3, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 2, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 4, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 4, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 4, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcde"), 6, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 0, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 0, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 0, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 0, 11, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 1, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 1, 8, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 1, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 1, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 5, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 5, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 5, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 5, 4, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 5, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 5, 6, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 9, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 9, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 9, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghij"), 11, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 0, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 0, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 0, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 0, 19, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 0, 20, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 0, 21, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 1, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 1, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 1, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 1, 18, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 1, 19, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 1, 20, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 10, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 10, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 10, 5, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 10, 9, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 10, 10, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 10, 11, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 19, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 19, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 19, 2, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 20, 0, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 20, 1, 0)));
    TC_ASSERT_FUNC((test(S("abcdefghijklmnopqrst"), 21, 0, S("abcdefghijklmnopqrst"), 21, 0, 0)));
    return 0;
}

template<class S>
static int test55()
{
    TC_ASSERT_FUNC((test_npos(S(""), 0, 0, S(""), 0, 0)));
    TC_ASSERT_FUNC((test_npos(S(""), 0, 0, S("abcde"), 0, -5)));
    TC_ASSERT_FUNC((test_npos(S("abcde"), 0, 0, S("abcdefghij"), 0, -10)));
    TC_ASSERT_FUNC((test_npos(S("abcde"), 0, 0, S("abcdefghij"), 1, -9)));
    TC_ASSERT_FUNC((test_npos(S("abcde"), 0, 0, S("abcdefghij"), 5, -5)));
    return 0;
}

int tc_libcxx_strings_string_compare_size_size_string_size_size(void)
{
    {
    typedef std::string S;
    TC_ASSERT_FUNC((test0<S>()));
    TC_ASSERT_FUNC((test1<S>()));
    TC_ASSERT_FUNC((test2<S>()));
    TC_ASSERT_FUNC((test3<S>()));
    TC_ASSERT_FUNC((test4<S>()));
    TC_ASSERT_FUNC((test5<S>()));
    TC_ASSERT_FUNC((test6<S>()));
    TC_ASSERT_FUNC((test7<S>()));
    TC_ASSERT_FUNC((test8<S>()));
    TC_ASSERT_FUNC((test9<S>()));
    TC_ASSERT_FUNC((test10<S>()));
    TC_ASSERT_FUNC((test11<S>()));
    TC_ASSERT_FUNC((test12<S>()));
    TC_ASSERT_FUNC((test13<S>()));
    TC_ASSERT_FUNC((test14<S>()));
    TC_ASSERT_FUNC((test15<S>()));
    TC_ASSERT_FUNC((test16<S>()));
    TC_ASSERT_FUNC((test17<S>()));
    TC_ASSERT_FUNC((test18<S>()));
    TC_ASSERT_FUNC((test19<S>()));
    TC_ASSERT_FUNC((test20<S>()));
    TC_ASSERT_FUNC((test21<S>()));
    TC_ASSERT_FUNC((test22<S>()));
    TC_ASSERT_FUNC((test23<S>()));
    TC_ASSERT_FUNC((test24<S>()));
    TC_ASSERT_FUNC((test25<S>()));
    TC_ASSERT_FUNC((test26<S>()));
    TC_ASSERT_FUNC((test27<S>()));
    TC_ASSERT_FUNC((test28<S>()));
    TC_ASSERT_FUNC((test29<S>()));
    TC_ASSERT_FUNC((test30<S>()));
    TC_ASSERT_FUNC((test31<S>()));
    TC_ASSERT_FUNC((test32<S>()));
    TC_ASSERT_FUNC((test33<S>()));
    TC_ASSERT_FUNC((test34<S>()));
    TC_ASSERT_FUNC((test35<S>()));
    TC_ASSERT_FUNC((test36<S>()));
    TC_ASSERT_FUNC((test37<S>()));
    TC_ASSERT_FUNC((test38<S>()));
    TC_ASSERT_FUNC((test39<S>()));
    TC_ASSERT_FUNC((test40<S>()));
    TC_ASSERT_FUNC((test41<S>()));
    TC_ASSERT_FUNC((test42<S>()));
    TC_ASSERT_FUNC((test43<S>()));
    TC_ASSERT_FUNC((test44<S>()));
    TC_ASSERT_FUNC((test45<S>()));
    TC_ASSERT_FUNC((test46<S>()));
    TC_ASSERT_FUNC((test47<S>()));
    TC_ASSERT_FUNC((test48<S>()));
    TC_ASSERT_FUNC((test49<S>()));
    TC_ASSERT_FUNC((test50<S>()));
    TC_ASSERT_FUNC((test51<S>()));
    TC_ASSERT_FUNC((test52<S>()));
    TC_ASSERT_FUNC((test53<S>()));
    TC_ASSERT_FUNC((test54<S>()));
    TC_ASSERT_FUNC((test55<S>()));
    }
    TC_SUCCESS_RESULT();
    return 0;
}
